@@ -138,18 +138,13 @@ set_toggle_property (GpdsXInput *xinput, GtkToggleButton *button, GpdsMousePrope
 {
     GError *error = NULL;
     gboolean active;
-    const gchar *property_name;
-    gint format_type;
     gint properties[1];
 
     active = gtk_toggle_button_get_active(button);
-    property_name = gpds_mouse_xinput_get_name(property);
-    format_type = gpds_mouse_xinput_get_format_type(property);
 
     properties[0] = active ? 1 : 0;
     gpds_xinput_set_int_properties(xinput,
-                                   property_name,
-                                   format_type,
+                                   property,
                                    &error,
                                    properties,
                                    1);
@@ -164,18 +159,13 @@ set_spin_property (GpdsXInput *xinput, GtkSpinButton *button, GpdsMouseProperty
 {
     GError *error = NULL;
     gdouble value;
-    const gchar *property_name;
-    gint format_type;
     gint properties[1];
 
     value = gtk_spin_button_get_value(button);
-    property_name = gpds_mouse_xinput_get_name(property);
-    format_type = gpds_mouse_xinput_get_format_type(property);
 
     properties[0] = (gint)value;
     gpds_xinput_set_int_properties(xinput,
-                                   property_name,
-                                   format_type,
+                                   property,
                                    &error,
                                    properties,
                                    1);
@@ -197,47 +187,59 @@ set_widget_sensitivity (GtkBuilder *builder,
                              gtk_toggle_button_get_active(button));
 }
 
-static void
-cb_middle_button_emulation_toggled (GtkToggleButton *button, gpointer user_data)
-{
-    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);
-    GtkBuilder *builder;
-    gboolean enable;
-
-    builder = gpds_ui_get_builder(GPDS_UI(user_data));
-
-    set_toggle_property(ui->xinput, button, GPDS_MOUSE_MIDDLE_BUTTON_EMULATION);
-
-    enable = gtk_toggle_button_get_active(button);
-    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_MOUSE_MIDDLE_BUTTON_EMULATION_KEY, enable);
-    set_widget_sensitivity(builder, "middle_button_emulation_box", button);
+#define DEFINE_EMULATION_BUTTON_TOGGLED_CALLBACK(name, KEY_NAME, box_name)          \
+static void                                                                         \
+cb_ ## name ## _toggled (GtkToggleButton *button,                                   \
+                         gpointer user_data)                                        \
+{                                                                                   \
+    GtkBuilder *builder;                                                            \
+    gboolean enable;                                                                \
+    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);                                     \
+    set_toggle_property(ui->xinput, button, GPDS_MOUSE_ ## KEY_NAME);               \
+    enable = gtk_toggle_button_get_active(button);                                  \
+    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_MOUSE_ ## KEY_NAME ## _KEY, enable);   \
+    builder = gpds_ui_get_builder(GPDS_UI(user_data));                              \
+    set_widget_sensitivity(builder, box_name, button);                              \
 }
 
+DEFINE_EMULATION_BUTTON_TOGGLED_CALLBACK(middle_button_emulation, MIDDLE_BUTTON_EMULATION, "middle_button_emulation_box")
+DEFINE_EMULATION_BUTTON_TOGGLED_CALLBACK(wheel_emulation, WHEEL_EMULATION, "wheel_emulation_box")
+
 static void
-cb_wheel_emulation_toggled (GtkToggleButton *button, gpointer user_data)
+cb_wheel_emulation_button_changed (GtkComboBox *combo, gpointer user_data)
 {
+    gint properties[1];
+    GtkTreeIter iter;
+    GObject *list_store;
+    GValue value = {0};
+    GError *error = NULL;
     GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);
     GtkBuilder *builder;
-    gboolean enable;
 
-    builder = gpds_ui_get_builder(GPDS_UI(user_data));
+    if (!gtk_combo_box_get_active_iter(combo, &iter))
+        return;
 
-    set_toggle_property(ui->xinput, button, GPDS_MOUSE_WHEEL_EMULATION);
+    builder = gpds_ui_get_builder(GPDS_UI(ui));
+    list_store = gtk_builder_get_object(builder, "wheel_emulation_button_list_store");
+    gtk_tree_model_get_value(GTK_TREE_MODEL(list_store),
+                             &iter,
+                             0,
+                             &value);
+    properties[0] = g_value_get_int(&value);
+    g_value_unset(&value);
 
-    enable = gtk_toggle_button_get_active(button);
-    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_MOUSE_WHEEL_EMULATION_KEY, enable);
-    set_widget_sensitivity(builder, "wheel_emulation_box", button);
-}
+    if (!gpds_xinput_set_int_properties(ui->xinput,
+                                        GPDS_MOUSE_WHEEL_EMULATION_BUTTON,
+                                        &error,
+                                        properties,
+                                        1)) {
+        if (error) {
+            show_error(error);
+            g_error_free(error);
+        }
+    }
 
-static void
-cb_wheel_emulation_button_value_changed (GtkSpinButton *button, gpointer user_data)
-{
-    gdouble value;
-    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);
-    set_spin_property(ui->xinput, button, GPDS_MOUSE_WHEEL_EMULATION_BUTTON);
-
-    value = gtk_spin_button_get_value(button);
-    gpds_ui_set_gconf_int(GPDS_UI(ui), GPDS_MOUSE_WHEEL_EMULATION_BUTTON_KEY, (gint)value);
+    gpds_ui_set_gconf_int(GPDS_UI(ui), GPDS_MOUSE_WHEEL_EMULATION_BUTTON_KEY, properties[0]);
 }
 
 static void
@@ -247,13 +249,11 @@ set_scroll_axes_property (GpdsMouseUI *ui)
     GtkToggleButton *button;
     GError *error = NULL;
     gboolean active;
-    const gchar *property_name;
-    gint format_type;
     gint properties[4];
 
     builder = gpds_ui_get_builder(GPDS_UI(ui));
 
-    button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "wheel_emulation_vertical"));
+    button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "wheel_emulation_horizontal"));
     active = gtk_toggle_button_get_active(button);
     if (active) {
         properties[0] = 6;
@@ -263,7 +263,7 @@ set_scroll_axes_property (GpdsMouseUI *ui)
         properties[1] = 0;
     }
 
-    button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "wheel_emulation_horizontal"));
+    button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "wheel_emulation_vertical"));
     active = gtk_toggle_button_get_active(button);
     if (active) {
         properties[2] = 4;
@@ -273,12 +273,8 @@ set_scroll_axes_property (GpdsMouseUI *ui)
         properties[3] = 0;
     }
 
-    property_name = gpds_mouse_xinput_get_name(GPDS_MOUSE_WHEEL_EMULATION_AXES);
-    format_type = gpds_mouse_xinput_get_format_type(GPDS_MOUSE_WHEEL_EMULATION_AXES);
-
     gpds_xinput_set_int_properties(ui->xinput,
-                                   property_name,
-                                   format_type,
+                                   GPDS_MOUSE_WHEEL_EMULATION_AXES,
                                    &error,
                                    properties,
                                    4);
@@ -288,60 +284,36 @@ set_scroll_axes_property (GpdsMouseUI *ui)
     }
 }
 
-static void
-cb_wheel_emulation_vertical_toggled (GtkToggleButton *button, gpointer user_data)
-{
-    gboolean enable;
-    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);
-    set_scroll_axes_property(ui);
-
-    enable = gtk_toggle_button_get_active(button);
-    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_MOUSE_WHEEL_EMULATION_Y_AXIS_KEY, enable);
+#define DEFINE_WHEEL_EMULATION_SCROLL_BUTTON_TOGGLED_CALLBACK(name, KEY_NAME)       \
+static void                                                                         \
+cb_ ## name ## _toggled (GtkToggleButton *button,                                   \
+                         gpointer user_data)                                        \
+{                                                                                   \
+    gboolean enable;                                                                \
+    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);                                     \
+    set_scroll_axes_property(ui);                                                   \
+    enable = gtk_toggle_button_get_active(button);                                  \
+    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_MOUSE_ ## KEY_NAME ## _KEY, enable);   \
 }
 
-static void
-cb_wheel_emulation_horizontal_toggled (GtkToggleButton *button, gpointer user_data)
-{
-    gboolean enable;
-    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);
-    set_scroll_axes_property(ui);
+DEFINE_WHEEL_EMULATION_SCROLL_BUTTON_TOGGLED_CALLBACK(wheel_emulation_vertical, WHEEL_EMULATION_Y_AXIS)
+DEFINE_WHEEL_EMULATION_SCROLL_BUTTON_TOGGLED_CALLBACK(wheel_emulation_horizontal, WHEEL_EMULATION_X_AXIS)
 
-    enable = gtk_toggle_button_get_active(button);
-    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_MOUSE_WHEEL_EMULATION_X_AXIS_KEY, enable);
+#define DEFINE_SPIN_BUTTON_VALUE_CHANGED_CALLBACK(name, NAME)                       \
+static void                                                                         \
+cb_ ## name ## _value_changed (GtkSpinButton *button,                               \
+                               gpointer user_data)                                  \
+{                                                                                   \
+    gdouble value;                                                                  \
+    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);                                     \
+    set_spin_property(ui->xinput, button, GPDS_MOUSE_ ## NAME);                     \
+    value = gtk_spin_button_get_value(button);                                      \
+    gpds_ui_set_gconf_int(GPDS_UI(ui), GPDS_MOUSE_ ## NAME ## _KEY, (gint)value);   \
 }
 
-static void
-cb_wheel_emulation_timeout_value_changed (GtkSpinButton *button, gpointer user_data)
-{
-    gdouble time;
-    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);
-    set_spin_property(ui->xinput, button, GPDS_MOUSE_WHEEL_EMULATION_TIMEOUT);
-
-    time = gtk_spin_button_get_value(button);
-    gpds_ui_set_gconf_int(GPDS_UI(ui), GPDS_MOUSE_WHEEL_EMULATION_TIMEOUT_KEY, (gint)time);
-}
-
-static void
-cb_wheel_emulation_inertia_value_changed (GtkSpinButton *button, gpointer user_data)
-{
-    gdouble inertia;
-    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);
-    set_spin_property(ui->xinput, button, GPDS_MOUSE_WHEEL_EMULATION_INERTIA);
-
-    inertia = gtk_spin_button_get_value(button);
-    gpds_ui_set_gconf_int(GPDS_UI(ui), GPDS_MOUSE_WHEEL_EMULATION_INERTIA_KEY, (gint)inertia);
-}
-
-static void
-cb_middle_button_timeout_value_changed (GtkSpinButton *button, gpointer user_data)
-{
-    gdouble time;
-    GpdsMouseUI *ui = GPDS_MOUSE_UI(user_data);
-    set_spin_property(ui->xinput, button, GPDS_MOUSE_MIDDLE_BUTTON_TIMEOUT);
-
-    time = gtk_spin_button_get_value(button);
-    gpds_ui_set_gconf_int(GPDS_UI(ui), GPDS_MOUSE_MIDDLE_BUTTON_TIMEOUT_KEY, (gint)time);
-}
+DEFINE_SPIN_BUTTON_VALUE_CHANGED_CALLBACK(wheel_emulation_timeout, WHEEL_EMULATION_TIMEOUT)
+DEFINE_SPIN_BUTTON_VALUE_CHANGED_CALLBACK(wheel_emulation_inertia, WHEEL_EMULATION_INERTIA)
+DEFINE_SPIN_BUTTON_VALUE_CHANGED_CALLBACK(middle_button_timeout, MIDDLE_BUTTON_TIMEOUT)
 
 static void
 setup_signals (GpdsUI *ui, GtkBuilder *builder)
@@ -358,7 +330,7 @@ setup_signals (GpdsUI *ui, GtkBuilder *builder)
     CONNECT(middle_button_timeout, value_changed);
     CONNECT(wheel_emulation, toggled);
     CONNECT(wheel_emulation_timeout, value_changed);
-    CONNECT(wheel_emulation_button, value_changed);
+    CONNECT(wheel_emulation_button, changed);
     CONNECT(wheel_emulation_inertia, value_changed);
     CONNECT(wheel_emulation_vertical, toggled);
     CONNECT(wheel_emulation_horizontal, toggled);
@@ -367,13 +339,13 @@ setup_signals (GpdsUI *ui, GtkBuilder *builder)
 }
 
 static gboolean
-get_integer_properties (GpdsXInput *xinput, const gchar *property_name,
+get_integer_properties (GpdsXInput *xinput, gint property_enum,
                         gint **values, gulong *n_values)
 {
     GError *error = NULL;
 
     if (!gpds_xinput_get_int_properties(xinput,
-                                        property_name,
+                                        property_enum,
                                         &error,
                                         values, n_values)) {
         if (error) {
@@ -398,11 +370,8 @@ set_integer_property_from_preference (GpdsMouseUI *ui,
     gint *values;
     gulong n_values;
     gint value;
-    const gchar *property_name;
 
-    property_name = gpds_mouse_xinput_get_name(property);
-
-    if (!get_integer_properties(ui->xinput, property_name,
+    if (!get_integer_properties(ui->xinput, property,
                                 &values, &n_values)) {
         return;
     }
@@ -426,12 +395,9 @@ set_boolean_property_from_preference (GpdsMouseUI *ui,
     gint *values;
     gulong n_values;
     gboolean enable;
-    const gchar *property_name;
     gchar *box_name;
 
-    property_name = gpds_mouse_xinput_get_name(property);
-
-    if (!get_integer_properties(ui->xinput, property_name,
+    if (!get_integer_properties(ui->xinput, property,
                                 &values, &n_values)) {
         return;
     }
@@ -456,11 +422,9 @@ set_scroll_axes_property_from_preference (GpdsMouseUI *ui,
     gint *values;
     gulong n_values;
     gboolean horizontal_enable = FALSE, vertical_enable = FALSE;
-    const gchar *property_name;
 
-    property_name = gpds_mouse_xinput_get_name(GPDS_MOUSE_WHEEL_EMULATION_AXES);
-
-    if (!get_integer_properties(ui->xinput, property_name,
+    if (!get_integer_properties(ui->xinput, 
+                                GPDS_MOUSE_WHEEL_EMULATION_AXES,
                                 &values, &n_values)) {
         return;
     }
@@ -483,15 +447,13 @@ set_scroll_axes_property_from_preference (GpdsMouseUI *ui,
 static void
 setup_num_buttons (GpdsUI *ui)
 {
-    GObject *spin;
-    gshort num_buttons;
+    GObject *list_store;
+    gshort num_buttons, i;
     GError *error = NULL;
     GtkBuilder *builder;
-    GtkAdjustment *adjustment;
 
     builder = gpds_ui_get_builder(ui);
 
-    spin = gtk_builder_get_object(builder, "wheel_emulation_button");
     num_buttons = gpds_xinput_utils_get_device_num_buttons(gpds_ui_get_device_name(ui),
                                                            &error);
     if (error) {
@@ -499,8 +461,78 @@ setup_num_buttons (GpdsUI *ui)
         g_error_free(error);
         return;
     }
-    adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
-    gtk_adjustment_set_upper(adjustment, num_buttons - 1);
+
+    list_store = gtk_builder_get_object(builder, "wheel_emulation_button_list_store");
+
+    for (i = num_buttons -1; i > 0; i--) {
+        GtkTreeIter iter;
+        gtk_list_store_prepend(GTK_LIST_STORE(list_store), &iter);
+        gtk_list_store_set(GTK_LIST_STORE(list_store), &iter, 0, i, -1);
+    }
+}
+
+static gboolean
+each_tree_model_iter (GtkTreeModel *model,
+                      GtkTreePath *path,
+                      GtkTreeIter *iter,
+                      gpointer data)
+{
+    GValue value = {0};
+    gint *list_index = data;
+    gint int_value;
+    gtk_tree_model_get_value(model,
+                             iter,
+                             0,
+                             &value);
+    int_value = g_value_get_int(&value);
+    g_value_unset(&value);
+
+    if (int_value == *list_index)  {
+        gint *indices;
+        indices = gtk_tree_path_get_indices(path);
+        if (indices)
+            *list_index = indices[0];
+        return TRUE;
+    }
+    return FALSE;
+}
+
+static void
+set_wheel_emulation_button_combo_state (GpdsMouseUI *ui, gint button)
+{
+    GObject *list_store;
+    GtkComboBox *combo;
+    GtkBuilder *builder;
+    gint list_index = button;
+
+    builder = gpds_ui_get_builder(GPDS_UI(ui));
+
+    list_store = gtk_builder_get_object(builder, "wheel_emulation_button_list_store");
+    gtk_tree_model_foreach(GTK_TREE_MODEL(list_store),
+                           each_tree_model_iter, &list_index);
+
+    combo = GTK_COMBO_BOX(gtk_builder_get_object(builder, "wheel_emulation_button"));
+    gtk_combo_box_set_active(combo, list_index);
+}
+
+static void
+set_wheel_emulation_button_property_from_preference (GpdsMouseUI *ui)
+{
+    gint *values;
+    gulong n_values;
+    gint button;
+
+    if (!get_integer_properties(ui->xinput,
+                                GPDS_MOUSE_WHEEL_EMULATION_BUTTON,
+                                &values, &n_values)) {
+        return;
+    }
+
+    if (!gpds_ui_get_gconf_int(GPDS_UI(ui), GPDS_MOUSE_WHEEL_EMULATION_BUTTON_KEY, &button))
+        button = values[0];
+    set_wheel_emulation_button_combo_state(ui, button);
+
+    g_free(values);
 }
 
 static void
@@ -518,12 +550,9 @@ setup_current_values (GpdsUI *ui, GtkBuilder *builder)
                                          GPDS_MOUSE_WHEEL_EMULATION_KEY,
                                          builder,
                                          "wheel_emulation");
-    set_integer_property_from_preference(mouse_ui,
-                                         GPDS_MOUSE_WHEEL_EMULATION_BUTTON,
-                                         GPDS_MOUSE_WHEEL_EMULATION_BUTTON_KEY,
-                                         builder,
-                                         "wheel_emulation_button");
     setup_num_buttons(ui);
+    set_wheel_emulation_button_property_from_preference(mouse_ui);
+
     set_integer_property_from_preference(mouse_ui,
                                          GPDS_MOUSE_MIDDLE_BUTTON_TIMEOUT,
                                          GPDS_MOUSE_MIDDLE_BUTTON_TIMEOUT_KEY,
@@ -574,7 +603,7 @@ build (GpdsUI  *ui, GError **error)
     }
 
     gpds_ui_set_gconf_string(ui, GPDS_GCONF_DEVICE_TYPE_KEY, "mouse");
-    GPDS_MOUSE_UI(ui)->xinput = gpds_xinput_new(gpds_ui_get_device_name(ui));
+    GPDS_MOUSE_UI(ui)->xinput = gpds_mouse_xinput_new(gpds_ui_get_device_name(ui));
 
     setup_current_values(ui, builder);
     setup_signals(ui, builder);
