@@ -151,8 +151,7 @@ set_toggle_property (GpdsXInput *xinput, GtkToggleButton *button, GpdsTouchpadPr
     properties[0] = gtk_toggle_button_get_active(button) ? 1 : 0;
 
     if (!gpds_xinput_set_int_properties(xinput,
-                                        gpds_touchpad_xinput_get_name(property),
-                                        gpds_touchpad_xinput_get_format_type(property),
+                                        property,
                                         &error,
                                         properties,
                                         1)) {
@@ -164,7 +163,7 @@ set_toggle_property (GpdsXInput *xinput, GtkToggleButton *button, GpdsTouchpadPr
 }
 
 static void
-set_edge_scroll_toggle_property (GpdsXInput *xinput, GtkBuilder *builder)
+set_edge_scrolling_toggle_property (GpdsXInput *xinput, GtkBuilder *builder)
 {
     GError *error = NULL;
     GObject *object;
@@ -178,11 +177,11 @@ set_edge_scroll_toggle_property (GpdsXInput *xinput, GtkBuilder *builder)
     properties[1] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object)) ? 1 :0;
     set_widget_sensitivity(builder, "horizontal_scrolling_box", properties[1]);
 
-    properties[2] = 0;
+    object = gtk_builder_get_object(builder, "continuous_edge_scrolling");
+    properties[2] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object)) ? 1 :0;
 
     if (!gpds_xinput_set_int_properties(xinput,
-                                        gpds_touchpad_xinput_get_name(GPDS_TOUCHPAD_EDGE_SCROLLING),
-                                        gpds_touchpad_xinput_get_format_type(GPDS_TOUCHPAD_EDGE_SCROLLING),
+                                        GPDS_TOUCHPAD_EDGE_SCROLLING,
                                         &error,
                                         properties,
                                         3)) {
@@ -193,6 +192,34 @@ set_edge_scroll_toggle_property (GpdsXInput *xinput, GtkBuilder *builder)
     }
 }
 
+static void
+set_two_finger_scrolling_toggle_property (GpdsXInput *xinput, GtkBuilder *builder)
+{
+    GError *error = NULL;
+    GObject *object;
+    gint properties[2];
+
+    object = gtk_builder_get_object(builder, "two_finger_vertical_scrolling");
+    properties[0] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object)) ? 1 :0;
+
+    object = gtk_builder_get_object(builder, "two_finger_horizontal_scrolling");
+    properties[1] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object)) ? 1 :0;
+
+    object = gtk_builder_get_object(builder, "continuous_edge_scrolling");
+    properties[2] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object)) ? 1 :0;
+
+    if (!gpds_xinput_set_int_properties(xinput,
+                                        GPDS_TOUCHPAD_TWO_FINGER_SCROLLING,
+                                        &error,
+                                        properties,
+                                        2)) {
+        if (error) {
+            show_error(error);
+            g_error_free(error);
+        }
+    }
+}
+
 static void
 set_range_property (GpdsXInput *xinput, GtkRange *range, GpdsTouchpadProperty property)
 {
@@ -201,8 +228,7 @@ set_range_property (GpdsXInput *xinput, GtkRange *range, GpdsTouchpadProperty pr
 
     properties[0] = (gint)gtk_range_get_value(range);
     if (!gpds_xinput_set_int_properties(xinput,
-                                        gpds_touchpad_xinput_get_name(property),
-                                        gpds_touchpad_xinput_get_format_type(property),
+                                        property,
                                         &error,
                                         properties,
                                         1)) {
@@ -227,8 +253,7 @@ set_scrolling_distance_range_property (GpdsXInput *xinput, GtkBuilder *builder)
     properties[1] = (gint)gtk_range_get_value(GTK_RANGE(object));
 
     if (!gpds_xinput_set_int_properties(xinput,
-                                        gpds_touchpad_xinput_get_name(GPDS_TOUCHPAD_SCROLLING_DISTANCE),
-                                        gpds_touchpad_xinput_get_format_type(GPDS_TOUCHPAD_SCROLLING_DISTANCE),
+                                        GPDS_TOUCHPAD_SCROLLING_DISTANCE,
                                         &error,
                                         properties,
                                         2)) {
@@ -248,8 +273,7 @@ set_circular_scrolling_trigger_property (GpdsTouchpadUI *ui, GpdsTouchpadCircula
     properties[0] = trigger;
 
     if (!gpds_xinput_set_int_properties(ui->xinput,
-                                        gpds_touchpad_xinput_get_name(GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER),
-                                        gpds_touchpad_xinput_get_format_type(GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER),
+                                        GPDS_TOUCHPAD_CIRCULAR_SCROLLING_TRIGGER,
                                         &error,
                                         properties,
                                         1)) {
@@ -299,36 +323,6 @@ cb_circular_scrolling_toggled (GtkToggleButton *button, gpointer user_data)
     gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_TOUCHPAD_CIRCULAR_SCROLLING_KEY, check);
 }
 
-static void
-cb_vertical_scrolling_toggled (GtkToggleButton *button, gpointer user_data)
-{
-    GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
-    GtkBuilder *builder;
-    gboolean check;
-
-    builder = gpds_ui_get_builder(GPDS_UI(user_data));
-
-    set_edge_scroll_toggle_property(ui->xinput, builder);
-
-    check = gtk_toggle_button_get_active(button);
-    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_TOUCHPAD_VERTICAL_SCROLLING_KEY, check);
-}
-
-static void
-cb_horizontal_scrolling_toggled (GtkToggleButton *button, gpointer user_data)
-{
-    GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);
-    GtkBuilder *builder;
-    gboolean check;
-
-    builder = gpds_ui_get_builder(GPDS_UI(user_data));
-
-    set_edge_scroll_toggle_property(ui->xinput, builder);
-
-    check = gtk_toggle_button_get_active(button);
-    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_TOUCHPAD_HORIZONTAL_SCROLLING_KEY, check);
-}
-
 static void
 cb_vertical_scrolling_scale_value_changed (GtkRange *range, gpointer user_data)
 {
@@ -359,6 +353,41 @@ cb_horizontal_scrolling_scale_value_changed (GtkRange *range, gpointer user_data
     gpds_ui_set_gconf_int(GPDS_UI(ui), GPDS_TOUCHPAD_VERTICAL_SCROLLING_DISTANCE_KEY, (gint)distance);
 }
 
+#define DEFINE_EDGE_SCROLLING_CALLBACK(type, TYPE)                                          \
+static void                                                                                 \
+cb_ ## type ## _scrolling_toggled (GtkToggleButton *button,                                 \
+                                   gpointer user_data)                                      \
+{                                                                                           \
+    GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);                                       \
+    GtkBuilder *builder;                                                                    \
+    gboolean check;                                                                         \
+    builder = gpds_ui_get_builder(GPDS_UI(user_data));                                      \
+    set_edge_scrolling_toggle_property(ui->xinput, builder);                                \
+    check = gtk_toggle_button_get_active(button);                                           \
+    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_TOUCHPAD_ ## TYPE ## _SCROLLING_KEY, check);   \
+}
+
+DEFINE_EDGE_SCROLLING_CALLBACK(vertical, VERTICAL)
+DEFINE_EDGE_SCROLLING_CALLBACK(horizontal, HORIZONTAL)
+DEFINE_EDGE_SCROLLING_CALLBACK(continuous_edge, CONTINUOUS_EDGE)
+
+#define DEFINE_TWO_FINGER_SCROLLING_CALLBACK(direction, DIRECTION)                                          \
+static void                                                                                                 \
+cb_two_finger_ ## direction ## _scrolling_toggled (GtkToggleButton *button,                                 \
+                                                   gpointer user_data)                                      \
+{                                                                                                           \
+    GpdsTouchpadUI *ui = GPDS_TOUCHPAD_UI(user_data);                                                       \
+    GtkBuilder *builder;                                                                                    \
+    gboolean check;                                                                                         \
+    builder = gpds_ui_get_builder(GPDS_UI(user_data));                                                      \
+    set_two_finger_scrolling_toggle_property(ui->xinput, builder);                                          \
+    check = gtk_toggle_button_get_active(button);                                                           \
+    gpds_ui_set_gconf_bool(GPDS_UI(ui), GPDS_TOUCHPAD_TWO_FINGER_ ## DIRECTION ## _SCROLLING_KEY, check);   \
+}
+
+DEFINE_TWO_FINGER_SCROLLING_CALLBACK(vertical, VERTICAL)
+DEFINE_TWO_FINGER_SCROLLING_CALLBACK(horizontal, HORIZONTAL)
+
 static void
 set_circular_scrolling_trigger_button_state (GpdsTouchpadUI *ui, 
                                              GpdsTouchpadCircularScrollingTrigger trigger)
@@ -486,8 +515,7 @@ cb_touchpad_use_type_changed (GtkComboBox *combo, gpointer user_data)
 
     properties[0] = gtk_combo_box_get_active(combo);
     if (!gpds_xinput_set_int_properties(ui->xinput,
-                                        gpds_touchpad_xinput_get_name(GPDS_TOUCHPAD_OFF),
-                                        gpds_touchpad_xinput_get_format_type(GPDS_TOUCHPAD_OFF),
+                                        GPDS_TOUCHPAD_OFF,
                                         &error,
                                         properties,
                                         1)) {
@@ -516,9 +544,12 @@ setup_signals (GpdsUI *ui, GtkBuilder *builder)
     CONNECT(faster_tapping_check, toggled);
     CONNECT(circular_scrolling, toggled);
     CONNECT(vertical_scrolling, toggled);
+    CONNECT(continuous_edge_scrolling, toggled);
     CONNECT(vertical_scrolling_scale, value_changed);
     CONNECT(horizontal_scrolling, toggled);
     CONNECT(horizontal_scrolling_scale, value_changed);
+    CONNECT(two_finger_vertical_scrolling, toggled);
+    CONNECT(two_finger_horizontal_scrolling, toggled);
 
     /* cirlular scrolling trigger */
     CONNECT(trigger_top_toggle, button_press_event);
@@ -541,7 +572,7 @@ get_integer_properties (GpdsXInput *xinput, GpdsTouchpadProperty property,
     GError *error = NULL;
 
     if (!gpds_xinput_get_int_properties(xinput,
-                                        gpds_touchpad_xinput_get_name(property),
+                                        property,
                                         &error,
                                         values, n_values)) {
         if (error) {
@@ -606,8 +637,8 @@ set_boolean_property_from_preference (GpdsTouchpadUI *ui,
 }
 
 static void
-set_edge_scroll_property_from_preference (GpdsTouchpadUI *ui,
-                                          GtkBuilder *builder)
+set_edge_scrolling_property_from_preference (GpdsTouchpadUI *ui,
+                                             GtkBuilder *builder)
 {
     GObject *object;
     gint *values;
@@ -636,6 +667,43 @@ set_edge_scroll_property_from_preference (GpdsTouchpadUI *ui,
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), enable);
     set_widget_sensitivity(builder, "horizontal_scrolling_box", enable);
 
+    if (!gpds_ui_get_gconf_bool(GPDS_UI(ui), GPDS_TOUCHPAD_CONTINUOUS_EDGE_SCROLLING_KEY, &enable))
+        enable = (values[0] == 1);
+    object = gtk_builder_get_object(builder, "continuous_edge_scrolling");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), enable);
+
+    g_free(values);
+}
+
+static void
+set_two_finger_scrolling_property_from_preference (GpdsTouchpadUI *ui,
+                                                   GtkBuilder *builder)
+{
+    GObject *object;
+    gint *values;
+    gulong n_values;
+    gboolean enable;
+
+    if (!get_integer_properties(ui->xinput, GPDS_TOUCHPAD_TWO_FINGER_SCROLLING,
+                                &values, &n_values)) {
+        return;
+    }
+
+    if (n_values != 2) {
+        g_free(values);
+        return;
+    }
+
+    if (!gpds_ui_get_gconf_bool(GPDS_UI(ui), GPDS_TOUCHPAD_TWO_FINGER_VERTICAL_SCROLLING_KEY, &enable))
+        enable = (values[0] == 1);
+    object = gtk_builder_get_object(builder, "two_finger_vertical_scrolling");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), enable);
+
+    if (!gpds_ui_get_gconf_bool(GPDS_UI(ui), GPDS_TOUCHPAD_TWO_FINGER_HORIZONTAL_SCROLLING_KEY, &enable))
+        enable = (values[0] == 1);
+    object = gtk_builder_get_object(builder, "two_finger_horizontal_scrolling");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(object), enable);
+
     g_free(values);
 }
 
@@ -745,9 +813,10 @@ setup_current_values (GpdsUI *ui, GtkBuilder *builder)
                                          builder,
                                          "faster_tapping_check");
     set_circular_scrolling_property_from_preference(touchpad_ui, builder);
-    set_edge_scroll_property_from_preference(touchpad_ui, builder);
+    set_edge_scrolling_property_from_preference(touchpad_ui, builder);
     set_scroll_distance_property_from_preference(touchpad_ui, builder);
     set_circular_scrolling_trigger_property_from_preference(touchpad_ui, builder);
+    set_two_finger_scrolling_property_from_preference(touchpad_ui, builder);
 
     set_touchpad_use_type_property_from_preference(touchpad_ui, builder);
 }
@@ -781,7 +850,7 @@ build (GpdsUI  *ui, GError **error)
     }
 
     gpds_ui_set_gconf_string(ui, GPDS_GCONF_DEVICE_TYPE_KEY, "touchpad");
-    GPDS_TOUCHPAD_UI(ui)->xinput = gpds_xinput_new(gpds_ui_get_device_name(ui));
+    GPDS_TOUCHPAD_UI(ui)->xinput = gpds_touchpad_xinput_new(gpds_ui_get_device_name(ui));
 
     setup_current_values(ui, builder);
     setup_signals(ui, builder);
