@@ -21,6 +21,7 @@
 #define __GPDS_GCONF_H__
 
 #include <glib.h>
+#include <gconf/gconf-client.h>
 
 G_BEGIN_DECLS
 
@@ -29,6 +30,16 @@ G_BEGIN_DECLS
 
 const gchar *gpds_gconf_get_key_from_path (const gchar *path);
 
+gboolean     gpds_gconf_get_boolean       (GConfClient *gconf,
+                                           const gchar *key,
+                                           gboolean *value);
+gboolean     gpds_gconf_get_int           (GConfClient *gconf,
+                                           const gchar *key,
+                                           gboolean *value);
+gboolean     gpds_gconf_get_string        (GConfClient *gconf,
+                                           const gchar *key,
+                                           const gchar **value);
+
 G_END_DECLS
 
 #endif /* __GPDS_GCONF_H__ */
