@@ -28,6 +28,7 @@
 #include <gconf/gconf-client.h>
 #include "gpds-module.h"
 #include "gpds-gconf.h"
+#include "gpds-xinput-utils.h"
 
 static GList *uis = NULL;
 
@@ -205,7 +206,17 @@ gpds_ui_is_available (GpdsUI *ui, GError **error)
 
     g_return_val_if_fail(GPDS_IS_UI(ui), FALSE);
 
+    if (!gpds_xinput_utils_exist_device(gpds_ui_get_device_name(ui))) {
+        g_set_error(error,
+                    GPDS_XINPUT_UTILS_ERROR,
+                    GPDS_XINPUT_UTILS_ERROR_NO_DEVICE,
+                    _("No %s found."), 
+                    gpds_ui_get_device_name(ui));
+        return FALSE;
+    }
+
     klass = GPDS_UI_GET_CLASS(ui);
+
     return (klass->is_available) ? klass->is_available(ui, error) : FALSE;
 }
 
@@ -293,7 +304,6 @@ gpds_ui_set_gconf_bool (GpdsUI *ui, const gchar *key, gboolean value)
 gboolean
 gpds_ui_get_gconf_bool (GpdsUI *ui, const gchar *key, gboolean *value)
 {
-    GConfValue *gconf_value;
     gchar *gconf_key;
     gboolean exist_value = FALSE;
     GpdsUIPriv *priv;
@@ -302,12 +312,7 @@ gpds_ui_get_gconf_bool (GpdsUI *ui, const gchar *key, gboolean *value)
 
     priv = GPDS_UI_GET_PRIVATE(ui);
     gconf_key = build_gconf_key(ui, key);
-    gconf_value = gconf_client_get(priv->gconf, gconf_key, NULL);
-    if (gconf_value) {
-        *value = gconf_value_get_bool(gconf_value);
-        gconf_value_free(gconf_value);
-        exist_value = TRUE;
-    }
+    exist_value =gpds_gconf_get_boolean(priv->gconf, gconf_key, value);
     g_free(gconf_key);
 
     return exist_value;
@@ -330,7 +335,6 @@ gpds_ui_set_gconf_int (GpdsUI *ui, const gchar *key, gint value)
 gboolean
 gpds_ui_get_gconf_int (GpdsUI *ui, const gchar *key, gint *value)
 {
-    GConfValue *gconf_value;
     gchar *gconf_key;
     gboolean exist_value = FALSE;
     GpdsUIPriv *priv;
@@ -339,12 +343,7 @@ gpds_ui_get_gconf_int (GpdsUI *ui, const gchar *key, gint *value)
 
     priv = GPDS_UI_GET_PRIVATE(ui);
     gconf_key = build_gconf_key(ui, key);
-    gconf_value = gconf_client_get(priv->gconf, gconf_key, NULL);
-    if (gconf_value) {
-        *value = gconf_value_get_int(gconf_value);
-        gconf_value_free(gconf_value);
-        exist_value = TRUE;
-    }
+    exist_value =gpds_gconf_get_int(priv->gconf, gconf_key, value);
     g_free(gconf_key);
 
     return exist_value;
@@ -367,7 +366,6 @@ gpds_ui_set_gconf_string (GpdsUI *ui, const gchar *key, const gchar *value)
 gboolean
 gpds_ui_get_gconf_string (GpdsUI *ui, const gchar *key, const gchar **value)
 {
-    GConfValue *gconf_value;
     gchar *gconf_key;
     gboolean exist_value = FALSE;
     GpdsUIPriv *priv;
@@ -376,12 +374,7 @@ gpds_ui_get_gconf_string (GpdsUI *ui, const gchar *key, const gchar **value)
 
     priv = GPDS_UI_GET_PRIVATE(ui);
     gconf_key = build_gconf_key(ui, key);
-    gconf_value = gconf_client_get(priv->gconf, gconf_key, NULL);
-    if (gconf_value) {
-        *value = gconf_value_get_string(gconf_value);
-        gconf_value_free(gconf_value);
-        exist_value = TRUE;
-    }
+    exist_value =gpds_gconf_get_string(priv->gconf, gconf_key, value);
     g_free(gconf_key);
 
     return exist_value;
