@@ -7,10 +7,13 @@ void test_exist_device (void);
 void test_get_float_atom (void);
 void test_get_device_info (void);
 void test_open_device (void);
+void test_get_device_num_buttons (void);
 
 static GError *error;
 static XDevice *device;
 
+#define DEVICE_NAME "Macintosh mouse button emulation"
+
 void
 setup (void)
 {
@@ -43,15 +46,28 @@ void
 test_get_device_info (void)
 {
     XDeviceInfo *device_info = NULL; 
-    device_info = gpds_xinput_utils_get_device_info("Macintosh mouse button emulation");
+    device_info = gpds_xinput_utils_get_device_info(DEVICE_NAME);
     cut_assert(device_info);
 }
 
 void
 test_open_device (void)
 {
-    device = gpds_xinput_utils_open_device ("Macintosh mouse button emulation", &error);
+    device = gpds_xinput_utils_open_device(DEVICE_NAME, &error);
     cut_assert(device);
+
+    gcut_assert_error(error);
+}
+
+void
+test_get_device_num_buttons (void)
+{
+    gshort n_buttons;
+
+    n_buttons = gpds_xinput_utils_get_device_num_buttons(DEVICE_NAME, &error);
+    gcut_assert_error(error);
+
+    cut_assert_equal_int(32, n_buttons);
 }
 
 /*
